@@ -1,4 +1,4 @@
-# Copyright 2015 The TensorFlow Authors. All Rights Reserved.
+# Copyright 2017 The TensorFlow Authors. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -94,13 +94,11 @@ class ScatterNdTest(test.TestCase):
                         tf_scatter,
                         vtype,
                         itype,
-                        use_gpu,
                         repeat_indices=False):
     np.random.seed(8)
     ref_shapes = [(3, 6), (3, 6), (3, 6, 9), (3, 6, 9), (3, 6, 9), (3, 6, 9)]
     indices_shapes = [(2,), (2, 2), (2,), (2, 2), (2, 3), (2, 3, 3)]
-    # TODO(apassos): re-enable when GPU support is working again.
-    with self.test_session(use_gpu=False):
+    with self.test_session(use_gpu=True):
       for ref_shape, indices_shape in zip(ref_shapes, indices_shapes):
         num_updates = indices_shape[0]
         ixdim = indices_shape[-1]
@@ -144,8 +142,48 @@ class ScatterNdTest(test.TestCase):
   def _VariableRankTests(self, np_scatter, tf_scatter):
     for vtype in (np.float32, np.float64):
       for itype in (np.int32, np.int64):
-        for use_gpu in (False, True):
-          self._VariableRankTest(np_scatter, tf_scatter, vtype, itype, use_gpu)
+        self._VariableRankTest(np_scatter, tf_scatter, vtype, itype)
+
+  def testSimple(self):
+    indices = constant_op.constant([[4], [3], [1], [7]], dtype=dtypes.int32)
+    updates = constant_op.constant([9, 10, 11, 12], dtype=dtypes.float32)
+    ref = variables.Variable([0, 0, 0, 0, 0, 0, 0, 0], dtype=dtypes.float32)
+    expected = np.array([0, 11, 0, 10, 9, 0, 0, 12])
+    scatter = state_ops.scatter_nd_update(ref, indices, updates)
+    init = variables.global_variables_initializer()
+
+    with self.test_session(use_gpu=True) as sess:
+      sess.run(init)
+      result = sess.run(scatter)
+      self.assertAllClose(result, expected)
+
+  def testSimple2(self):
+    indices = constant_op.constant([[1, 0], [1, 1]], dtype=dtypes.int32)
+    updates = constant_op.constant([11., 12.], dtype=dtypes.float32)
+    ref = variables.Variable(
+        [[0., 0.], [0., 0.], [0., 0.]], dtype=dtypes.float32)
+    expected = np.array([[0., 0.], [11., 12.], [0., 0.]])
+    scatter = state_ops.scatter_nd_update(ref, indices, updates)
+    init = variables.global_variables_initializer()
+
+    with self.test_session(use_gpu=True) as sess:
+      sess.run(init)
+      result = sess.run(scatter)
+      self.assertAllClose(result, expected)
+
+  def testSimple3(self):
+    indices = constant_op.constant([[1]], dtype=dtypes.int32)
+    updates = constant_op.constant([[11., 12.]], dtype=dtypes.float32)
+    ref = variables.Variable(
+        [[0., 0.], [0., 0.], [0., 0.]], dtype=dtypes.float32)
+    expected = np.array([[0., 0.], [11., 12.], [0., 0.]])
+    scatter = state_ops.scatter_nd_update(ref, indices, updates)
+    init = variables.global_variables_initializer()
+
+    with self.test_session(use_gpu=True) as sess:
+      sess.run(init)
+      result = sess.run(scatter)
+      self.assertAllClose(result, expected)
 
   def testVariableRankUpdate(self):
     self._VariableRankTests(_NumpyUpdate, state_ops.scatter_nd_update)
@@ -167,14 +205,8 @@ class ScatterNdTest(test.TestCase):
   def _ScatterRepeatIndicesTest(self, np_scatter, tf_scatter):
     for vtype in (np.float32, np.float64):
       for itype in (np.int32, np.int64):
-        for use_gpu in (False, True):
-          self._VariableRankTest(
-              np_scatter,
-              tf_scatter,
-              vtype,
-              itype,
-              use_gpu,
-              repeat_indices=True)
+        self._VariableRankTest(
+            np_scatter, tf_scatter, vtype, itype, repeat_indices=True)
 
   def testScatterRepeatIndices(self):
     """This tests scatter_add using indices that repeat."""
@@ -378,8 +410,8 @@ class ScatterNdTest(test.TestCase):
     indices = constant_op.constant([[0, 1]] * num_updates, dtype=dtypes.int32)
     updates = constant_op.constant(update_values, dtype=dtypes.float64)
 
-    exepected_result = np.zeros([2, 2], dtype=np.float64)
-    exepected_result[0, 1] = np.sum(update_values)
+    expected_result = np.zeros([2, 2], dtype=np.float64)
+    expected_result[0, 1] = np.sum(update_values)
 
     scatter = state_ops.scatter_nd_add(ref, indices, updates)
     init = variables.global_variables_initializer()
@@ -387,7 +419,7 @@ class ScatterNdTest(test.TestCase):
     with session.Session() as sess:
       sess.run(init)
       result = sess.run(scatter)
-      assert np.allclose(result, exepected_result)
+      assert np.allclose(result, expected_result)
 
   # TODO(fpmc): Re-enable this test when gpu_pip test actually runs on a GPU.
   def _disabledTestScatterOutOfRangeGpu(self):
@@ -424,6 +456,34 @@ class ScatterNdTest(test.TestCase):
       val = array_ops.scatter_nd(indices, values, shape).eval()
     self.assertAllClose([np.sum(values)], val)
 
+  def testSmokeScatterNdBatch2DSliceDim2(self):
+    with self.test_session():
+      indices = array_ops.zeros([3, 5, 2], dtype=dtypes.int32)
+      values = array_ops.zeros([3, 5, 7])
+      shape = [4, 6, 7]
+      array_ops.scatter_nd(indices, values, shape).eval()
+
+  def testSmokeScatterNdBatch1DSliceDim2(self):
+    with self.test_session():
+      indices = array_ops.zeros([0, 2], dtype=dtypes.int32)
+      values = array_ops.zeros([0, 7])
+      shape = [4, 6, 7]
+      array_ops.scatter_nd(indices, values, shape).eval()
+
+  def testSmokeScatterNdBatch1DSliceDim3ShapeRank7(self):
+    with self.test_session():
+      indices = array_ops.zeros([1, 3], dtype=dtypes.int32)
+      values = array_ops.zeros([1, 6, 7, 8, 9])
+      shape = [3, 4, 5, 6, 7, 8, 9]
+      array_ops.scatter_nd(indices, values, shape).eval()
+
+  def testSmokeScatterNdBatch2DSliceDim3ShapeRank7(self):
+    with self.test_session():
+      indices = array_ops.zeros([1, 2, 3], dtype=dtypes.int32)
+      values = array_ops.zeros([1, 2, 6, 7, 8, 9])
+      shape = [3, 4, 5, 6, 7, 8, 9]
+      array_ops.scatter_nd(indices, values, shape).eval()
+
 
 if __name__ == "__main__":
   test.main()
